function MayronUI_CastBars:OnEnable()

	-- defaults:
	local C = {
		["Interruptable"] = {r = 0, g = 0.45, b = 0.73, a = 0.6},
		["Non-interruptable"] = {r = 1, g = 0.47, b = 0, a = 0.6},
		["LatencyBar"] = {r = 0, g = 0.2, b = 0.41, a = 0.8},
		["Background"] = {r = 0, g = 0, b = 0, a = 0.5},
		["Border"] = {r = 0, g = 0, b = 0, a = 1},
		["Success"] = {r = 0, g = 1, b = 0, a = 1},
		["Failed"] = {r = 1, g = 0, b = 0, a = 1},
		["Text"] = {r = 1, g = 1, b = 1, a = 1},
		["Timer"] = {r = 0.03, g = 0.78, b = 097, a = 1},
	}
	
	local ticks = {
		-- Warlock
		[GetSpellInfo(689) or ""] = 1, -- Drain Life
		[GetSpellInfo(1120) or ""] = 2, -- Drain Soul
		[GetSpellInfo(755) or ""] = 1, -- Health Funnel
		[GetSpellInfo(5740) or ""] = 2, -- Rain of Fire
		[GetSpellInfo(1949) or ""] = 1, -- Hellfire
		[GetSpellInfo(103103) or ""] = 1, -- Malefic Grasp
		[GetSpellInfo(108371) or ""] = 1, -- Harvest Life
		-- Druid
		[GetSpellInfo(740) or ""] = 2, -- Tranquility
		[GetSpellInfo(16914) or ""] = 1, -- Hurricane
		[GetSpellInfo(127663) or ""] = 1, -- Astral Communion
		-- Priest
		[GetSpellInfo(47540) or ""] = 1, -- Penance
		[GetSpellInfo(15407) or ""] = 1, -- Mind Flay
		[GetSpellInfo(129197) or ""] = 1, -- Mind Flay (Insanity)
		[GetSpellInfo(48045) or ""] = 1, -- Mind Sear
		[GetSpellInfo(64843) or ""] = 2, -- Divine Hymn
		[GetSpellInfo(64901) or ""] = 2, -- Hymn of Hope
		-- Mage
		[GetSpellInfo(10) or ""] = 1, -- Blizzard
		[GetSpellInfo(5143) or ""] = 0.4, -- Arcane Missiles
		[GetSpellInfo(12051) or ""] = 2, -- Evocation
		-- Monk
		[GetSpellInfo(117952) or ""] = 1, -- Crackling Jade Lightning
		[GetSpellInfo(115175) or ""] = 1, -- Soothing Mist
		[GetSpellInfo(113656) or ""] = 1, -- Fists of Fury
		[GetSpellInfo(115294) or ""] = 1, -- Mana Tea
	}
	local tickTextures = {};				
	function MUI_CastBar_Load(self, unit)
		self:RegisterUnitEvent("UNIT_SPELLCAST_START", unit);
		self:RegisterUnitEvent("UNIT_SPELLCAST_STOP", unit);
		self:RegisterUnitEvent("UNIT_SPELLCAST_FAILED", unit);
		self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
		self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
		self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
		self:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
		self:RegisterEvent("PLAYER_ENTERING_WORLD");
		
		if ( unit == "player" ) then			
			self:RegisterEvent("UNIT_SPELLCAST_SENT");
		elseif ( unit == "target" ) then
			self:RegisterEvent("PLAYER_TARGET_CHANGED");
		elseif ( unit == "focus" ) then
			self:RegisterEvent("PLAYER_FOCUS_CHANGED");
		elseif ( unit == "targettarget" ) then
			self:RegisterEvent("UNIT_TARGET"); -- does not instantly trigger
			self.affectedUnit = "target";
		end
		
		-- default property values:
		self.unit = unit;
		self.casting = nil;
		self.channeling = nil;
		self.beginFading = nil;
		
		-- More default settings:
		self.updateColors = true;
		_G[self:GetName().."Text"].showText = true;		
		_G[self:GetName().."Timer"].showDuration = true;
		_G[self:GetName().."Timer"].showRemaining = true;

		self.holdTime = 0;
	end

	function MUI_CastBar_OnEvent(self, event, ...)
		local unit = self.unit;

		--print(unit.." : "..event.." : "..arg1)
		MayronUI.db.profile.CastBars.Appearance.toggle = nil;
				
		if ( (event == "PLAYER_ENTERING_WORLD") or (event == "PLAYER_TARGET_CHANGED") or (event == "PLAYER_FOCUS_CHANGED") or (event == "UNIT_TARGET" ) ) then		
			if (event == "PLAYER_ENTERING_WORLD") then
				self:Hide();				
			end
			local nameChannel = UnitChannelInfo(unit);
			local nameSpell = UnitCastingInfo(unit);
			if (not (nameChannel or nameSpell)) then
				self:Hide();
				return; -- return if changed and not casting anything.
			end
		end				
			
		local barSpark = _G[self:GetName().."Spark"];
		local nameText = _G[self:GetName().."Text"];

		if ( event == "UNIT_SPELLCAST_SENT" ) then
			self.latency = GetTime();
		elseif ( event == "UNIT_SPELLCAST_START" ) then
			local name, nameSubtext, text, texture, startTime, endTime, _, castID, notInterruptible = UnitCastingInfo(unit);
			if ( not name ) then -- No Casting Bar active
				--self:Hide();
				return;
			end
			for _, tick in ipairs(tickTextures) do
				tick:Hide()
			end	
			if ( self.latency and unit == "player" ) then
				local perc = ((GetTime()) - self.latency);
				self.latency = perc * self:GetWidth();
				if (self.latency > self:GetWidth()) then
					self.latency = 0; -- should fix the extremely large latency bug.
				end
				if ( self.latency == 0 ) then self.latency = 1; end
			end	
			
			self:Show();
			self:SetAlpha(1.0);
			self.castID = castID;
			if ( nameText and nameText.showText ) then
				nameText:SetText(text);
			end
			
			if ( notInterruptible ) then
				self:SetStatusBarColor(C["Non-interruptable"].r, C["Non-interruptable"].g, C["Non-interruptable"].b, C["Non-interruptable"].a);				
			else
				self:SetStatusBarColor(C["Interruptable"].r, C["Interruptable"].g, C["Interruptable"].b, C["Interruptable"].a);
			end
			
			-- Sets up correct values:
			self.value = (GetTime() - (startTime / 1000));
			self.maxValue = (endTime - startTime) / 1000;
			self:SetMinMaxValues(0, self.maxValue);
			MUI_CastBar_Fading(self, true, 1); -- should enable casting and stop channeling
		elseif ( event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_CHANNEL_STOP") then
			if not (self:IsVisible()) then 
				--self:Hide();
			end; -- alpha was set to 0
			if ( (self.casting and event == "UNIT_SPELLCAST_STOP" and select(4, ...) == self.castID) or (self.channeling and event == "UNIT_SPELLCAST_CHANNEL_STOP") ) then
				self:SetValue(self.maxValue);
				self:SetStatusBarColor(C["Success"].r, C["Success"].g, C["Success"].b, C["Success"].a); -- successful color
				MUI_CastBar_Fading(self); -- stop casting		
			end
		elseif ( event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_INTERRUPTED" ) then
			if ( self:IsShown() and (self.casting and select(4, ...) == self.castID) ) then
				if ( nameText and nameText.showText ) then
					if ( event == "UNIT_SPELLCAST_FAILED" ) then
						nameText:SetText(FAILED);
					else
						nameText:SetText(INTERRUPTED);
					end
				end				
				self:SetValue(self.maxValue);
				self:SetStatusBarColor(C["Failed"].r, C["Failed"].g, C["Failed"].b, C["Failed"].a); -- failed color				
				MUI_CastBar_Fading(self); -- stop casting			
			end
		elseif ( event == "UNIT_SPELLCAST_DELAYED" ) then
			if ( self:IsShown() ) then
				local name, _, text, _, startTime, endTime, _ = UnitCastingInfo(unit);
				if ( not name ) then -- if there is no name, there is no bar					
					--self:Hide();
					return;	
				end
				self:Show();
				
				-- Spell delayed so redo Mix/Max values!
				self.value = (GetTime() - (startTime / 1000));
				self.maxValue = (endTime - startTime) / 1000;
				self:SetMinMaxValues(0, self.maxValue);				
			end
		elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" ) then
			self:SetStatusBarColor(C["Interruptable"].r, C["Interruptable"].g, C["Interruptable"].b, C["Interruptable"].a);			
			local name, _, text, _, startTime, endTime, _, notInterruptible = UnitChannelInfo(unit);
			if ( not name ) then
				-- if there is no name, there is no bar
				--self:Hide();
				return;
			end
			self:Show();
			self.latency = nil;
			
			self.value = ((endTime / 1000) - GetTime());
			self.maxValue = (endTime - startTime) / 1000;
			self:SetMinMaxValues(0, self.maxValue);
			
			-- Get tick info:
			if (self == MUI_PlayerCastBar) then
				local castingTime = 2500 / (1 + (GetCombatRatingBonus(CR_HASTE_SPELL) or 0) / 100);
				if (ticks[name]) then
					local tickDuration = (castingTime / 2500) * ticks[name];
					
					if (tickDuration) then
						local width = self:GetWidth();
						local delta = (tickDuration * width / self.maxValue);
						local tickIndex = 1;
						while (delta * tickIndex) < width do
							local tick;
							if (tickTextures[tickIndex]) then
								tick = tickTextures[tickIndex];
							else	
								tick = MUI_PlayerCastBar:CreateTexture(nil, "OVERLAY");						
								tick:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark");
								tick:SetVertexColor(1, 1, 1, 1);
								tick:SetBlendMode('ADD');
								tick:SetSize(24, MUI_PlayerCastBar:GetHeight() + 10);
								tinsert(tickTextures, tick);
							end						
							tick:SetPoint("CENTER", MUI_PlayerCastBar, "LEFT", delta * tickIndex, 0);
							tick:Show();
							tickIndex = tickIndex + 1;
						end
						for id, tick in ipairs(tickTextures) do
							if (id > tickIndex) then
								tick:Hide()
							end
						end
					end
				end
			end
			
			if ( nameText and nameText.showText ) then
				nameText:SetText(text);
			end
			
			self:SetAlpha(1.0);
			MUI_CastBar_Fading(self, 1, nil, 1);
			
		elseif ( event == "UNIT_SPELLCAST_CHANNEL_UPDATE" ) then
			if ( self:IsShown() ) then
				local name, _, text, _, startTime, endTime, _, notInterruptible = UnitChannelInfo(unit);
				if ( not name ) then return end
				self:Show();
				
				if ( notInterruptible ) then
					self:SetStatusBarColor(C["Non-interruptable"].r, C["Non-interruptable"].g, C["Non-interruptable"].b, C["Non-interruptable"].a);				
				else
					self:SetStatusBarColor(C["Interruptable"].r, C["Interruptable"].g, C["Interruptable"].b, C["Interruptable"].a);
				end
				
				self.value = ((endTime / 1000) - GetTime());
				self.maxValue = (endTime - startTime) / 1000;
				self:SetMinMaxValues(0, self.maxValue);
				
			end
		elseif ( event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE" ) then
			if ( select(8, UnitChannelInfo(unit)) ) then
				self:SetStatusBarColor(C["Non-interruptable"].r, C["Non-interruptable"].g, C["Non-interruptable"].b, C["Non-interruptable"].a);				
			else
				self:SetStatusBarColor(C["Interruptable"].r, C["Interruptable"].g, C["Interruptable"].b, C["Interruptable"].a);
			end
		end
	end

	function MUI_CastBar_OnUpdate(self, elapsed)
		if (MayronUI.db.profile.CastBars.Appearance.toggle) then return end
		local barSpark = _G[self:GetName().."Spark"];
		local timer = _G[self:GetName().."Timer"];		
		if ( self.casting ) then
			self.value = self.value + elapsed;
			
			if ( self.value >= self.maxValue ) then
				self:SetValue(self.maxValue);			
				return;
			end
			self:SetValue(self.value);
			
			if ( barSpark ) then
				local sparkPosition = (self.value / self.maxValue) * self:GetWidth();
				barSpark:SetPoint("CENTER", self, "LEFT", sparkPosition, 0);
			end
		elseif ( self.channeling ) then
			self.value = self.value - elapsed;
			if ( self.value <= 0 ) then
				self:SetValue(self.maxValue);
				return;
			end
			
			self:SetValue(self.value);
			if ( barSpark ) then
				local sparkPosition = (self.value / self.maxValue) * self:GetWidth();
				barSpark:SetPoint("CENTER", self, "LEFT", sparkPosition, 0);
			end
		elseif ( GetTime() < self.holdTime ) then
			return;
		elseif ( self.beginFading ) then -- fade out after hold time has ended.
			local alpha = self:GetAlpha() - 0.02;
			if ( alpha > 0 ) then
				self:SetAlpha(alpha);
				_G[self:GetName().."Text"]:SetText("");
				_G[self:GetName().."Timer"]:SetText("");
			else
				self.beginFading = nil;
				self.holdTime = 0;
				--self:Hide();
			end
		end
		if ( self.casting or self.channeling ) then
			local r = ( (timer.showRemaining) and (math.ceil((self.maxValue - self.value) * 10)) / 10 );
			local d = ( (timer.showDuration) and (math.ceil(self.maxValue * 10)) / 10 );
			if (not d) then d = ""; elseif (r) then d = " / "..tostring(d); end				
			if (not r) then r = ""; end			
			timer:SetText(r..d);
		else
			timer:SetText("");
		end
	end

	function MUI_CastBar_Fading(self, show, casting, channeling)
		local barSpark = _G[self:GetName().."Spark"];
		
		if (show) then -- Fade In
			if not (barSpark:IsShown()) then barSpark:Show(); end
			if ( _G[self:GetName().."Latency"] and self.latency and ( self:GetName() == "MUI_PlayerCastBar" ) ) then
				local latencyBar = _G[self:GetName().."Latency"];
				latencyBar:SetWidth(self.latency);
				latencyBar:Show()		
				self.latency = nil;
			else
				_G[self:GetName().."Latency"]:Hide()
			end
			
			self.casting = casting;
			self.channeling = channeling;
			return;
		else -- Fade Out
			if (barSpark:IsShown()) then barSpark:Hide(); end
			if (latencyBar and latencyBar:IsShown()) then latencyBar:Hide(); end
			self.holdTime = GetTime() + 0.4;
			self.casting = nil;
			self.channeling = nil;
			self.beginFading = 1;			
			return;
		end
	end
	
	-- Block Blizzard's Cast Bar:
	CastingBarFrame:UnregisterAllEvents();
	CastingBarFrame:Hide();
	
	if (not MayronUI.db.profile.Installed) then
		while not (next(MayronUI.db.profile.CastBars)) do
			MayronUI.db.profile.CastBars = {
				["Player"] = {Enabled = true, ["AnchorSUF"] = true}, 
				["Target"] = {Enabled = true, ["AnchorSUF"] = true},
				["TargetTarget"] = {},
				["Focus"] = {},
				Appearance = {},
			}
			break;
		end
	end
	if (not MayronUI.db.profile.CastBars.Appearance) then
		MayronUI.db.profile.CastBars.Appearance = {};
	end
	
	-- Positioning of Cast bars is done in the BottomUI.lua file!	
	function MayronUI:CastBarSetup(individual, setting)
		local bar;
		for barType, tbl in pairs(MayronUI.db.profile.CastBars) do
			if ( (individual == barType) or (not individual) ) then
				if (MayronUI.db.profile.CastBars[barType].Enabled) then
				
					-- load cast bar:
					if (not _G["MUI_"..barType.."CastBar"]) then
						bar = CreateFrame("StatusBar", "MUI_"..barType.."CastBar", UIParent, "MUI_CastBarTemplate");
						MUI_CastBar_Load(_G["MUI_"..barType.."CastBar"], barType:lower()); -- 2nd arg is UnitID
					else
						bar = _G["MUI_"..barType.."CastBar"];
					end
					
					-- import settings: (Should check which settigns have been altered and act on them)
					for str, value in pairs(MayronUI.db.profile.CastBars.Appearance) do	
						if ( (setting == str) or (not setting) ) then
							local mui = MayronUI.db.profile.CastBars.Appearance;
							local name, property = string.split("_", str);
							if (property) then
								local f = {
									["Size"] = function()
										local file, _, flags = _G[bar:GetName()..name]:GetFont();										
										_G[bar:GetName()..name]:SetFont(file, value, flags);
									end,
									["Field"] = function()	
										local timer = _G[bar:GetName().."Timer"];
										if (name == "Text") then
											_G[bar:GetName()..name].showText = value;
											if (value and MayronUI.db.profile.CastBars.Appearance.toggle) then  
												_G[bar:GetName().."Text"]:SetText(barType);											
											else
												_G[bar:GetName().."Text"]:SetText("")
											end
											return
										elseif (name == "Timer") then
											if (value) then
												timer.showDuration = (mui["Duration_Field"] == nil and true);
												timer.showRemaining = (mui["Remaining_Field"] == nil and true);
											else
												timer.showDuration = nil;
												timer.showRemaining = nil;
											end
										elseif (name == "Duration") then
											timer.showDuration = value;
										elseif (name == "Remaining") then
											timer.showRemaining = value;
										end										
										local r = ( (timer.showRemaining) and "95.5");
										local d = ( (timer.showDuration) and "100.0");
										if (not d) then d = ""; elseif (r) then d = " / "..d; end				
										if (not r) then r = ""; end			
										timer:SetText(r..d);
									end,
									["Font"] = function()
										local a, size, flags = _G[bar:GetName()..name]:GetFont();										
										_G[bar:GetName()..name]:SetFont(LibStub("LibSharedMedia-3.0"):Fetch("font", value), size, flags);
									end,
									["Type"] = function() 
										if (name == "Border") then
											local ins = mui["Border_Inset"] or 0;
											bar.border:SetBackdrop( { 
												edgeFile = LibStub("LibSharedMedia-3.0"):Fetch("border", (mui["Border_Type"] or "MUI_StatusBar") ), 
												edgeSize = (mui["Border_EdgeSize"] or 1),									  
											});
											bar.border:SetBackdropBorderColor(C["Border"].r, C["Border"].g, C["Border"].b, C["Border"].a);
											bar.border:SetPoint("TOPLEFT", bar, "TOPLEFT", -(ins), ins);
											bar.border:SetPoint("BOTTOMRIGHT", bar, "BOTTOMRIGHT", ins, -(ins));									
										elseif (name == "StatusBar") then										
											bar:SetStatusBarTexture(LibStub("LibSharedMedia-3.0"):Fetch("statusbar", value));
											_G[bar:GetName().."Latency"]:SetTexture(LibStub("LibSharedMedia-3.0"):Fetch("statusbar", value));
											bar.bg:SetTexture(LibStub("LibSharedMedia-3.0"):Fetch("statusbar", value));
										end
									end,
									["Color"] = function()								
										C[name].r, C[name].g, C[name].b, C[name].a = unpack(value);
										bar.updateColors = true;
									end,
									["Outline"] = function()
										local f = {"_", "OUTLINE, MONOCHROME", "OUTLINE", "THICKOUTLINE"}
										local file, size, _ = _G[bar:GetName()..name]:GetFont();
										if (f[value] ~= "_") then
											_G[bar:GetName()..name]:SetFont(file, size, f[value]);
										else
											_G[bar:GetName()..name]:SetFont(file, size)
										end
									end,
									["Move"] = function()										
										local p, rf, rp, x, y = _G[bar:GetName()..name]:GetPoint();
										x, y = (mui[name.."_X"] or x), (mui[name.."_Y"] or y);									
										_G[bar:GetName()..name]:SetPoint(p, rf, rp, tonumber(x), tonumber(y));
									end,
								}							
								if (property == "EdgeSize" or property == "Inset") then property = "Type" end
								if (property == "X" or property == "Y") then property = "Move" end
								f[property]();
							end
						end
					end
				
					
					-- Update colours after changing in GUI:
					if (bar.updateColors) then
						bar:SetStatusBarColor(C["Interruptable"].r, C["Interruptable"].g, C["Interruptable"].b, C["Interruptable"].a);
						_G[bar:GetName().."Latency"]:SetVertexColor(C["LatencyBar"].r, C["LatencyBar"].g, C["LatencyBar"].b, C["LatencyBar"].a);		
						_G[bar:GetName().."Timer"]:SetTextColor(C["Timer"].r, C["Timer"].g, C["Timer"].b, C["Timer"].a);
						bar.bg:SetVertexColor(C["Background"].r, C["Background"].g, C["Background"].b, C["Background"].a);
						bar.border:SetBackdropBorderColor(C["Border"].r, C["Border"].g, C["Border"].b, C["Border"].a);
						_G[bar:GetName().."Text"]:SetTextColor(C["Text"].r, C["Text"].g, C["Text"].b, C["Text"].a);
						bar.updateColors = nil;
					end
				end
			end
		end
		return bar;
	end	
	MayronUI:CastBarSetup();
end

if (IsAddOnLoaded("Quartz")) then
	Quartz3CastBarPlayer:SetFrameStrata("HIGH");
	Quartz3CastBarTarget:SetFrameStrata("HIGH");
	Quartz3CastBarFocus:SetFrameStrata("HIGH");
end