function MayronUI_ChatBox:OnEnable()
	local function print(...)
		DEFAULT_CHAT_FRAME:AddMessage(string.join(" ", MUI_PREFIX, tostringall(...)));
	end

	-- Global values:
	CHAT_FRAME_TAB_SELECTED_NOMOUSE_ALPHA = 1;
	CHAT_FRAME_TAB_NORMAL_NOMOUSE_ALPHA = 1;
	CHAT_FRAME_TAB_ALERTING_NOMOUSE_ALPHA = 1;
	CHAT_FRAME_TAB_SELECTED_MOUSEOVER_ALPHA = 1;
	CHAT_FRAME_TAB_NORMAL_MOUSEOVER_ALPHA = 1;
	CHAT_FRAME_TAB_ALERTING_MOUSEOVER_ALPHA = 1;
	CHAT_TAB_SHOW_DELAY = 1;
	CHAT_TAB_HIDE_DELAY = 1;
	CHAT_FRAME_FADE_TIME = 1;
	CHAT_FRAME_FADE_OUT_TIME = 1;
	DEFAULT_CHATFRAME_ALPHA = 0;
	
	local CHAT_TAB_POSITIONING = {["x"] = 16, ["y"] = 0};
	local layouts = {};
	
	CHAT_FONT_HEIGHTS = {
		[1] = 8,
		[2] = 9,
		[3] = 10,
		[4] = 11,
		[5] = 12,
		[6] = 13,
		[7] = 14,
		[8] = 15,
		[9] = 16,
		[10] = 17,
		[11] = 18,
	};
				
	local function UpdateTabColors()
		for i = 1, NUM_CHAT_WINDOWS do	
			local tab = _G["ChatFrame"..i.."Tab"];
			local cf = _G["ChatFrame"..i]
			if (tab and (cf == SELECTED_CHAT_FRAME)) then
				tab:GetFontString():SetTextColor(1,1,1,1)
			elseif (tab) then
				tab:GetFontString():SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 1)
			end
		end
	end
	
	-- Override Blizzard Functions:
	function FCFTab_UpdateColors() end
	
	function FCF_SetTabPosition(chatFrame)
		local cf = chatFrame:GetName();
		if (not chatFrame.isDocked) then
			_G[cf.."Tab"]:ClearAllPoints()
			if (IsCombatLog(chatFrame)) then				
				_G[cf.."Tab"]:SetPoint("BOTTOMLEFT", "CombatLogQuickButtonFrame_Custom", "TOPLEFT", 15, 10)
			else
				_G[cf.."Tab"]:SetPoint("BOTTOMLEFT", chatFrame, "TOPLEFT", 15, 10)
			end
		end
		ChatFrame1Tab:SetPoint("LEFT", CHAT_TAB_POSITIONING["x"], CHAT_TAB_POSITIONING["y"])
		_G[cf.."ButtonFrameMinimizeButton"]:Hide();
	end

	-- 3 Hooks to control repositioning of tabs:
	hooksecurefunc("FCFTab_OnUpdate", function()
		ChatFrame1Tab:SetPoint("LEFT", CHAT_TAB_POSITIONING["x"], CHAT_TAB_POSITIONING["y"])
	end)
	hooksecurefunc("FCF_DockUpdate", function()
		ChatFrame1Tab:SetPoint("LEFT", CHAT_TAB_POSITIONING["x"], CHAT_TAB_POSITIONING["y"])		
	end)
	hooksecurefunc("FCFDockScrollFrame_JumpToTab", function() 
		ChatFrame1Tab:SetPoint("LEFT", CHAT_TAB_POSITIONING["x"], CHAT_TAB_POSITIONING["y"])
	end)
	
	local LoadChatFrame;
	do
		function LoadChatFrame()		
			function string.color(text, color)
				return "|cff"..color..text.."|r"
			end
			function string.link(text, id, body, color)
				return "|H"..id..":"..tostring(body).."|h"..tostring(text):color(color or "ffffff").."|h"
			end			
			for i = 1, 20 do
				if _G["StaticPopup" .. i .. "EditBox"] then			
					_G["StaticPopup" .. i .. "EditBoxLeft"]:Hide()
					_G["StaticPopup" .. i .. "EditBoxMid"]:Hide()
					_G["StaticPopup" .. i .. "EditBoxRight"]:Hide()
				else break end
			end			
			StaticPopupDialogs["MUI_Link"] = {
				text = "|cff00ccffMayronUI|r\n(CTRL+C to Copy, CTRL+V to Paste)",
				button1 = "Close",
				hasEditBox = 1,
				maxLetters = 1024,
				editBoxWidth = 350,
				hideOnEscape = 1,
				timeout = 0,
				whileDead = 1,
				preferredIndex = 3, 
			}			
			function ChatFrame_OnHyperlinkShow(self, link, text, button)
				local something, value = link:match("(%a+):(.+)")		
				if ( something == "url" ) then
					local popup = StaticPopup_Show("MUI_Link")
					local editbox = _G[popup:GetName().."EditBox"]  
					editbox:SetText(value)
					editbox:SetFocus()
					editbox:HighlightText()
				else
					SetItemRef(link, text, button, self);
				end
			end
			
			-- Edit Box:
			local e = ChatFrame1EditBox;
			e.focusLeft.Show = e.focusLeft.Hide;
			e.focusRight.Show = e.focusRight.Hide;
			e.focusMid.Show = e.focusMid.Hide;
			
			ChatFrame1EditBoxLeft:Hide();
			ChatFrame1EditBoxMid:Hide();
			ChatFrame1EditBoxRight:Hide();
			e:SetBackdrop( { 
				bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background", 
				edgeFile = MUI_MEDIA.."Borders\\Skinner.tga", 
				edgeSize = 1, 
				insets = { left = 0, right = 0, top = 0, bottom = 0 }
			});
			e:SetBackdropColor(0,0,0, 1);
			e:SetHeight(28);
			
			hooksecurefunc("ChatEdit_UpdateHeader", function()
				local e = ChatFrame1EditBox;			
				local chatType = e:GetAttribute("chatType")
				local r, g, b = GetMessageTypeColor(chatType)
				e:SetBackdropBorderColor(r, g, b, 1)
			end);
			e = nil;

			hooksecurefunc("BNConversationButton_UpdateAttachmentPoint", function(self)
				if self:IsShown() then
					self:Hide()
				end
			end)
			
			hooksecurefunc("FCF_OpenTemporaryWindow", function()
				local chat = FCF_GetCurrentChatFrame();
				if (chat) then
					chat:SetClampRectInsets(0, 0, 0, 0);
				end
			end)
			
			local hidden = CreateFrame("FRAME")
			hidden:Hide()
			ChatFrameMenuButton:SetParent(hidden)
			FriendsMicroButton:SetParent(hidden)
			hidden:RegisterEvent("PLAYER_ENTERING_WORLD")
			hidden:SetScript("OnEvent", function(self)	
				self:UnregisterEvent("PLAYER_ENTERING_WORLD")
				UpdateTabColors()
			end)
			
			-- Chat Appearance Alterations:
			local function EditTabs(i)
				if not _G["ChatFrame"..i] then return end;				
				local cf = "ChatFrame"..i;				
				local btn = _G[cf.."ButtonFrameBottomButton"];
				local flash = _G[cf.."ButtonFrameBottomButtonFlash"];
				local tab = _G[cf.."Tab"];	
				
				local function f(url)				
					return string.link("["..url.."]", "url", url, "ffe29e")
				end
				
				if (i ~= 11) then
					if ( i ~= 2 ) then
						local chatFrame = _G[cf];
						chatFrame.f = _G[cf].AddMessage;
						chatFrame.AddMessage = function(self, text, ...)
							self:f(text:gsub("[wWhH][wWtT][wWtT][\46pP]%S+[^%p%s]", f), ...);
						end						
						chatFrame:HookScript("OnHyperLinkEnter", function(self, linkData)
							local linkType = string.split(":", linkData)
							if linkType == "item" or linkType == "spell" or linkType == "enchant" or linkType == "quest"
							or linkType == "talent" or linkType == "glyph" or linkType == "unit" or linkType == "achievement" then
								GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
								GameTooltip:SetHyperlink(linkData)
								GameTooltip:Show()
							end
						end)
						chatFrame:HookScript("OnHyperLinkLeave", function() GameTooltip:Hide() end)
					end
				end
				
				_G[cf.."TabSelectedLeft"]:SetParent(hidden);
				_G[cf.."TabSelectedMiddle"]:SetParent(hidden);
				_G[cf.."TabSelectedRight"]:SetParent(hidden);
				_G["ChatFrame"..i.."EditBox"]:SetAltArrowKeyMode(false)
				_G[cf.."ButtonFrameUpButton"]:SetParent(hidden)
				_G[cf.."ButtonFrameDownButton"]:SetParent(hidden)
				_G[cf .. "ButtonFrame"]:DisableDrawLayer("BACKGROUND")
				_G[cf .. "ButtonFrame"]:DisableDrawLayer("BORDER")
				
				_G[cf]:HookScript("OnMouseWheel", function(self, direction)
					if IsControlKeyDown() then
						if (direction == 1) then self:ScrollToTop() else self:ScrollToBottom() end
					elseif IsShiftKeyDown() then 
						if (direction == 1) then self:PageUp() else self:PageDown() end
					end
				end)
				
				btn:ClearAllPoints();
				btn:SetPoint("BOTTOM", tab, "BOTTOM", 0, -2);
				btn:SetSize(tab:GetWidth() - 10, 20)
				flash:SetAlpha(0.5);
				flash:SetTexture("Interface/BUTTONS/GRADBLUE.png");
				
				-- Remove textures
				btn:SetNormalTexture("");
				btn:SetHighlightTexture("");
				btn:SetPushedTexture("");			
				
				local fs = tab:GetFontString();
				fs:ClearAllPoints();
				fs:SetPoint("CENTER", tab, "CENTER");
				
				_G[cf.."TabLeft"]:Hide();
				_G[cf.."TabMiddle"]:Hide();
				_G[cf.."TabRight"]:Hide();
				_G[cf.."TabHighlightLeft"]:Hide();
				_G[cf.."TabHighlightMiddle"]:Hide();
				_G[cf.."TabHighlightRight"]:Hide();
				
				tab:SetHeight(16);
				tab:SetScript("OnEnter", function(self) 
					self:GetFontString():SetTextColor(1,1,1,1);
				end)
				
				local function tabHandler(self)
					local num = string.match((SELECTED_CHAT_FRAME:GetName()),"%d+");
					num = tonumber(num);
					for i = 1, NUM_CHAT_WINDOWS do
						if (_G["ChatFrame"..i]) then
							if (num == i) then
								_G["ChatFrame"..i.."Tab"]:GetFontString():SetTextColor(1,1,1,1);
							else
								_G["ChatFrame"..i.."Tab"]:GetFontString():SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 1)
							end
						end
					end					
				end
				tab:HookScript("OnLeave", function(self) tabHandler(self) end)
				tab:HookScript("OnClick", function(self)
					_G[cf]:ScrollToBottom();
					tabHandler(self);
				end)
			end -- End of EditTabs function
			
			for i = 1, NUM_CHAT_WINDOWS do -- Alters appearance of tabs
				EditTabs(i)
			end
			
			hooksecurefunc("PetBattleFrame_UpdateActionBarLayout", function()
				EditTabs(11)
			end)		
		end
	end
	LoadChatFrame()
	LoadChatFrame = nil;
	collectgarbage("collect");

	local function GetDBObject(addonStr) -- gets addon name
		local addon, dbstr, okay;		
		if (_G[addonStr]) then			
			addon = _G[addonStr];
			okay = true;
		end
		if (addon and not addon.db) then
			for dbname, tbl in pairs(addon) do				
				if (strfind(dbname, "db")) then					
					if (type(addon[dbname]) == "table") then
						
						if (addon[dbname].profile) then
							dbstr = dbname;
						end
					end
				end
			end		
			return addon[dbstr];
		elseif (addon and addon.db) then 
			return addon.db;
		end
	end	
	
	do
		local toggle = { -- to add a new function, enter it here and add to options menu
			["Character"] = function() ToggleCharacter("PaperDollFrame") end,
			["Bags"] = function() 
				if (ContainerFrame1:IsVisible()) then
					ToggleBackpack();
				else
					OpenAllBags();
				end
			end,
			["Friends"] = function() ToggleFriendsFrame() end,
			["Guild"] = function() 
				if (IsTrialAccount()) then
					print("Starter Edition accounts cannot perform that action.")
				else
					ToggleGuildFrame();
				end
			end,
			["Help Menu"] = function() ToggleHelpFrame() end,
			["PVP"] = function() 
				if (UnitLevel("player") < 10) then
					print("Requires level 10+ to view the PVP window.")
				else
					TogglePVPUI();
				end
			end,
			["Spell Book"] = function()
				ToggleFrame(SpellBookFrame)
				if ( (not MayronUI.db.profile.ReskinUI["Spell Book"]) and MayronUI_ReskinUI:IsEnabled() ) then
					MayronUI_ReskinUI.ToggleSpellBook()
				end
			end, -- fixed taint
			["Talents"] = function() 
				if (UnitLevel("player") < 10) then
					print("Requires level 10+ to view the Talents window.")
				else
					if (not _G["PlayerTalentFrame"]) then
						LoadAddOn("Blizzard_TalentUI");
					end
					ToggleFrame(PlayerTalentFrame);
				end
			end,
			["Achievements"] = function() 
				ToggleAchievementFrame();
			 end,
			["Glyphs"] = function() 
				if (UnitLevel("player") < 10) then
					print("Requires level 10+ to view the Glyphs window.")
				else
					ToggleGlyphFrame();
				end
			end,
			["Calendar"] = function() ToggleCalendar() end,
			["LFD"] = function() 
				if (UnitLevel("player") < 15) then
					print("Requires level 15+ to view the LFD window.")
				else
					ToggleLFDParentFrame();
				end
			 end,
			["Raid"] = function() ToggleRaidFrame() end,
			["Encounter Journal"] = function() ToggleEncounterJournal() end,
			["Mounts and Pets"] = function() 
				if (not _G["CollectionsJournal"]) then
					LoadAddOn("Blizzard_Collections");
				end
				ToggleFrame(CollectionsJournal); 
			end, -- fixed taint
			["Macros"] = function()
				if (not MacroFrame) then LoadAddOn("Blizzard_MacroUI") end ToggleFrame(MacroFrame);
			end,
			["Quest Log"] = function() ToggleFrame(WorldMapFrame) end, -- obsolete
			["World Map"] = function() ToggleFrame(WorldMapFrame) end,
			["Reputation"] = function() ToggleCharacter("ReputationFrame") end,
			["PVP Score"] = function() 
				if (not UnitInBattleground("player")) then
					print("Requires being inside a Battle Ground.")
				else
					ToggleWorldStateScoreFrame();
				end
			end,
			["Currency"] = function() ToggleCharacter("TokenFrame") end,
			[" "] = function() print("This button has not been assigned a function. To assign a function, visit the \"/mui config\" menu.") end,
			["Switch"] = function(self)	
				for id, layName in ipairs(layouts) do						
					if (MayronUI.db.profile.CurrentLayout == layName) then -- the next layout
						MayronUI.db.profile.CurrentLayout = layouts[id + 1] or layouts[1];
						self:SetText( " "..(MayronUI.db.profile.CurrentLayout:sub(1, 1):upper()) );
						print("|cff66CD00"..MayronUI.db.profile.CurrentLayout.." |r Layout Enabled")
						for layName, tbl in pairs(MayronUI.db.global.Layouts) do
							if (MayronUI.db.profile.CurrentLayout == layName) then -- the next layout
								for addon, profile in pairs(tbl) do
									local path = GetDBObject(addon)	
									if (path) then
										path:SetProfile(profile)  -- Switch all found AddOns.
									end
								end
								break;												
							end	
						end
						break;
					end	
				end
			end,
		}

		function MUI_LayoutButton_OnLoad(self)
			self:RegisterForClicks("LeftButtonDown", "RightButtonDown", "MiddleButtonDown")
			-- Inject default settings:
			while not (next(MayronUI.db.global.Layouts)) do
					MayronUI.db.global.Layouts = {
						["DPS"] = {
							["ShadowUF"] = "Default",
							["Grid"] = "Default",
						},
						["Healer"] = {
							["ShadowUF"] = "MayronUIH",
							["Grid"] = "MayronUIH",
						}
					}
				break;
			end
			if (not MayronUI.db.profile.CurrentLayout) then
				MayronUI.db.profile.CurrentLayout = "DPS";
			end
			
			-- Set up Chat Box properties here:
			local parent = self:GetParent()
			parent.name = parent:GetName();
			parent.id = tonumber(parent.name:match("%d+"));				
			for corner, id in pairs(MayronUI.db.profile.ChatBox) do
				if (id == parent.id) then									
					parent.corner = corner;
					break
				end
			end
				
			local value = MayronUI.db.profile.ChatFrame[parent.corner].functions[10]
			if (value == "Switch") then
				self:SetText( " "..(MayronUI.db.profile.CurrentLayout:sub(1, 1):upper()) );
				self:GetFontString():Show()
			else
				self:SetText(value);
				self:GetFontString():Hide()
			end
		end
		
		function MUI_ChatButtonOnClick(self, button, text) -- self is the button
			if ( (not IsAltKeyDown()) and (button == "LeftButton") ) then
				local fs = self:GetFontString();
				local text = text or fs:GetText();
				if (not toggle[text] or text == "Switch") then -- it's the switch button
					if (InCombatLockdown()) then 
						print("Cannot perform action in combat.");
						return 
					end
					if (#layouts == 0 ) then
						for layout, tbl in pairs(MayronUI.db.global.Layouts) do
							tinsert(layouts, layout);
						end	
					end						
					toggle["Switch"](self);					
				else
					toggle[text](self);
				end	
			elseif ( (IsAltKeyDown()) and (button == "LeftButton") and (self:GetName():find("LayoutButton")) ) then
				if (MayronUI.db.profile.ChatFrame.Tooltip) then
					MayronUI.db.profile.ChatFrame.Tooltip = nil;
					MUI_LayoutButton_OnEnter(self);
				else
					MayronUI.db.profile.ChatFrame.Tooltip = true;
					GameTooltip:Hide();
				end
			elseif (button == "RightButton") then
				if (not MUI_LayoutTool) then
					if (#layouts == 0 ) then
						for layout, tbl in pairs(MayronUI.db.global.Layouts) do -- Inserts Layouts into local table.
							tinsert(layouts, layout);
						end	
					end	
					local layoutTool = CreateFrame("Frame", "MUI_LayoutTool", UIParent, "MUI_LayoutToolTemplate")
				elseif (MUI_LayoutTool:IsShown()) then
					MUI_LayoutTool:Hide()
				else
					MUI_LayoutTool:Show()
				end
			elseif (button == "MiddleButton") then	
				ChatMenu:Show()
				ChatMenu:ClearAllPoints()
				ChatMenu:SetPoint("TOPLEFT", MUI_ChatFrame1LayoutButton, "TOPRIGHT", 8)
			end
			
		end		
	end
	
	function MUI_LayoutButton_OnEnter(self)
		if (MayronUI.db.profile.ChatFrame.Tooltip) then return end
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 8, -38)
		GameTooltip:SetText("MayronUI Layout Button");
		GameTooltip:AddDoubleLine("Left Click:", "Switch Layout / Custom Function", 1, 1, 1)
		GameTooltip:AddDoubleLine("Middle Click:", "Show Blizzard Chat Menu", 1, 1, 1)
		GameTooltip:AddDoubleLine("Right Click:", "Show Layout Config Tool", 1, 1, 1)
		GameTooltip:AddDoubleLine("ALT + Left Click:", "Toggle this Tooltip", 1, 0, 0, 1, 0, 0)
		GameTooltip:Show()
	end
	
	do
		local function IsModKeyDown(strKey) -- "C" for example.	
			if (strKey == "S") then -- The frame is either expand_retract, chatbuttons2 or chatbuttons3
				if (not IsControlKeyDown() and IsShiftKeyDown() and not IsAltKeyDown()) then return true end
			elseif (strKey == "C") then 
				if (IsControlKeyDown() and not IsShiftKeyDown() and not IsAltKeyDown()) then return true end
			elseif (strKey == "A") then
				if (not IsControlKeyDown() and not IsShiftKeyDown() and IsAltKeyDown()) then return true end
			elseif (strKey == "SC") then 
				if (IsControlKeyDown() and IsShiftKeyDown() and not IsAltKeyDown()) then return true end
			elseif (strKey == "SA") then 
				if (not IsControlKeyDown() and IsShiftKeyDown() and IsAltKeyDown()) then return true end
			elseif (strKey == "CA") then 
				if (IsControlKeyDown() and not IsShiftKeyDown() and IsAltKeyDown()) then return true end
			elseif (strKey == "SCA") then 
				if (IsControlKeyDown() and IsShiftKeyDown() and IsAltKeyDown()) then return true end
			end
		end

		function MUI_ChatButtonOnLoad(self)
			local id = self:GetID();
			self.state = id;

			local locator = tonumber(self:GetParent():GetName():match("%d+"));			

			for corner, id in pairs(MayronUI.db.profile.ChatBox) do
				if (id == locator) then					
					locator = corner;
					break;
				end
			end
			
			local value = MayronUI.db.profile.ChatFrame[locator].functions[id];
			if (value == "Switch") then
				self:GetFontString():SetText( " "..(MayronUI.db.profile.CurrentLayout:sub(1, 1):upper()) );
			else
				self:GetFontString():SetText(MayronUI.db.profile.ChatFrame[locator].functions[id]);
			end

			self:RegisterEvent('MODIFIER_STATE_CHANGED')
			self:SetScript("OnEvent", function(self)
				if (InCombatLockdown()) then return end
				if (IsModKeyDown(MayronUI.db.profile.ChatFrame[locator].ModKeys[1])) then
					self.state = id + 3;
				elseif (IsModKeyDown(MayronUI.db.profile.ChatFrame[locator].ModKeys[2])) then
					self.state = id + 6;
				else
					self.state = id;					
				end
				self:GetFontString():SetText(MayronUI.db.profile.ChatFrame[locator].functions[self.state]);	
			end)
			
			self:RegisterForClicks("LeftButtonDown", "RightButtonDown", "MiddleButtonDown")
			self:SetScript("OnClick", function(self, button)
				local text;
				if (button == "RightButton") then					
					text = MayronUI.db.profile.ChatFrame[locator].functions[id + 3];	
				elseif (button == "MiddleButton") then
					text = MayronUI.db.profile.ChatFrame[locator].functions[id + 6];	
				elseif (button == "LeftButton") then
					text = MayronUI.db.profile.ChatFrame[locator].functions[self.state];					
				end
				MUI_ChatButtonOnClick(self, "LeftButton", text);
			end)
		end
	end
	
	-- MayronUI Layout Tool:
	do
		local function GetAceAddOns()
			local addons = {}; -- Add additional Supported AddOns here
			if (IsAddOnLoaded("ShadowedUnitFrames")) then tinsert(addons, "ShadowUF") end
			if (IsAddOnLoaded("SLDataText")) then tinsert(addons, "SLDataText") end 
			for name, status in LibStub("AceAddon-3.0"):IterateAddonStatus() do
				local mod = strfind(name, "_");
				if ( (not mod) and status ) then				
					local path = GetDBObject(name);
					if (path) then
						tinsert(addons, name);
					end
				end
			end
			return addons;
		end

		local curLayout;
		function MUI_LayoutTool_OnLoad(self)
			self:RegisterForDrag("LeftButton");
			self.mainText:SetText("Here you can customise which AddOn/s should change to which Profile/s for each individual Layout.\nYou can also create additional Layouts if you want more than the default DPS/Healer Layouts (up to 5).");
			
			UIDropDownMenu_SetText(MUI_LayoutToolLayoutDropDown, curLayout);
			if (#layouts == 5) then
				MUI_LayoutToolCreateButton:Disable()
			elseif (#layouts == 1) then
				MUI_LayoutToolDeleteButton:Disable()
			end
		end
		
		-- Choose Layout Button:
		function MUI_LayoutDropDown_Initialize(self) 
			UIDropDownMenu_SetText(self, curLayout);
			
			for id, name in ipairs(layouts) do
				local info = UIDropDownMenu_CreateInfo();
				info.text = name;
				info.value = id;
				info.func = function() 
					UIDropDownMenu_SetText(self, layouts[id])
					curLayout = layouts[id];
					MUI_LayoutAddonScrollBar_Update();
				end
				UIDropDownMenu_AddButton(info);
			end
			
			_G[self:GetName().."Button"]:SetScript("OnClick", function(self)
				ToggleDropDownMenu(1, nil, MUI_LayoutToolLayoutDropDown, self, 0, 0);
				PlaySound("igMainMenuOptionCheckBoxOn");				
			end)
		end

		do
			function MUI_LayoutToolButton_OnClick(self, name)
				if ( (self:GetName()) == "MUI_LayoutToolCreateButton") then
					if (not StaticPopupDialogs["MUI_CreateLayout"]) then
						StaticPopupDialogs["MUI_CreateLayout"] = {
							text = "Name of New Layout:",
							button1 = "Confirm",
							button2 = "Cancel",
							timeout = 0,
							whileDead = true,
							hideOnEscape = true,
							hasEditBox  = true,
							preferredIndex = 3,
							OnShow = function (self)
								self.editBox:SetText( "Layout "..(#layouts + 1) )
							end,
							OnAccept = function(self)
								local text = self.editBox:GetText()
								if (MayronUI.db.global.Layouts[text]) then return end
								MayronUI.db.global.Layouts[text] = {};				
								curLayout = text;
								MayronUI.db.profile.CurrentLayout = text;
								tinsert(layouts, text);
								UIDropDownMenu_SetText(MUI_LayoutToolLayoutDropDown, curLayout);
								if (#layouts == 5) then
									MUI_LayoutToolCreateButton:Disable()
								elseif (#layouts > 1) then
									MUI_LayoutToolDeleteButton:Enable()
								end
								MUI_LayoutAddonScrollBar_Update();
							end,
						}
					end
					StaticPopup_Show("MUI_CreateLayout")
				elseif ( (self:GetName()) == "MUI_LayoutToolRenameButton") then
					if (not StaticPopupDialogs["MUI_RenameLayout"]) then
						StaticPopupDialogs["MUI_RenameLayout"] = {
						text = "Rename Layout:",
						button1 = "Confirm",
						button2 = "Cancel",
						timeout = 0,
						whileDead = true,
						hideOnEscape = true,
						hasEditBox  = true,
						preferredIndex = 3,
						OnAccept = function(self)
							local text = self.editBox:GetText()
							UIDropDownMenu_SetText(MUI_LayoutToolLayoutDropDown, text);
							MayronUI.db.global.Layouts[text] = MayronUI.db.global.Layouts[curLayout]; 
							MayronUI.db.global.Layouts[curLayout] = nil;
							for i, v in ipairs(layouts) do
								if (v == curLayout) then 
									layouts[i] = text;
									MayronUI.db.profile.CurrentLayout = text;
									curLayout = text;						
									break;
								end					
							end
						end,
						}
					end
					StaticPopup_Show("MUI_RenameLayout")
				elseif ( (self:GetName()) == "MUI_LayoutToolDeleteButton") then
					if (not StaticPopupDialogs["MUI_DeleteLayout"]) then
						StaticPopupDialogs["MUI_DeleteLayout"] = {						
							button1 = "Confirm",
							button2 = "Cancel",
							timeout = 0,
							whileDead = true,
							hideOnEscape = true,
							preferredIndex = 3,
							OnAccept = function(self)
								MayronUI.db.global.Layouts[curLayout] = nil; -- does not delete DPS or Healer Layouts!
								for i, v in ipairs(layouts) do
									if (v == curLayout) then 
										tremove(layouts, i);
										break;
									end					
								end				
								for layout, tbl in pairs(MayronUI.db.global.Layouts) do -- Inserts Layouts into local table.
									if (layout ~= curLayout) then
										MayronUI.db.profile.CurrentLayout = layout;
										curLayout = layout;
										break;
									end
								end					
								UIDropDownMenu_SetText(MUI_LayoutToolLayoutDropDown, curLayout);
								if (#layouts == 1) then
									MUI_LayoutToolDeleteButton:Disable()
								elseif (#layouts < 5) then
									MUI_LayoutToolCreateButton:Enable()
								end
								MUI_LayoutAddonScrollBar_Update();
							end,
						}
					end
					StaticPopupDialogs["MUI_DeleteLayout"].text = "Are you sure you want to delete Layout \""..curLayout.."\"?";
					StaticPopup_Show("MUI_DeleteLayout")
				elseif (name == "MUI_AceAddonDropDown") then
					if (not StaticPopupDialogs["MUI_NewProfileLayout"]) then -- new addon profile
						StaticPopupDialogs["MUI_NewProfileLayout"] = {
							text = "Create New Profile:",
							button1 = "Confirm",
							button2 = "Cancel",
							timeout = 0,
							whileDead = true,
							hideOnEscape = true,
							hasEditBox  = true,
							preferredIndex = 3,
							OnAccept = function(dialog)
								local text = dialog.editBox:GetText()								
								UIDropDownMenu_SetText(self, text)
								local path = GetDBObject(self.name)
								if (path) then
									local curProfile = path:GetCurrentProfile();
									path:SetProfile(text)
									path:SetProfile(curProfile)
									MayronUI.db.global.Layouts[curLayout][self.name] = text;
								end
							end,
						}
					end
					StaticPopup_Show("MUI_NewProfileLayout")
				end
			end
			
			local function MUI_AddonDropDown_Initialize(self) -- self: the drop-down frame
				local path = GetDBObject(self.name);
				if (path) then
					local length = #(path:GetProfiles());
					for id, name in ipairs(path:GetProfiles()) do	
						local info = UIDropDownMenu_CreateInfo();
						info.text = name;
						info.value = id;
						info.func = function() 
							UIDropDownMenu_SetText(self, name)
							MayronUI.db.global.Layouts[curLayout][self.name] = name;
						end
						UIDropDownMenu_AddButton(info);
						if (id == length) then
							length = nil;
							info = UIDropDownMenu_CreateInfo();
							info.text = YELLOW_FONT_COLOR_CODE.."< New Profile >";
							info.value = id + 1;
							info.func = function()
								MUI_LayoutToolButton_OnClick(self, "MUI_AceAddonDropDown")
							end
							UIDropDownMenu_AddButton(info);					
						end
					end
					_G[self:GetName().."Middle"]:SetWidth(180);			
					if (MayronUI.db.global.Layouts[curLayout][self.name]) then
						UIDropDownMenu_SetText(self, MayronUI.db.global.Layouts[curLayout][self.name])										
					else
						UIDropDownMenu_SetText(self, path:GetCurrentProfile())
					end
				end
			end
			
			local ADDONS;
			function MUI_LayoutToolAddons_OnLoad(self) -- First to be loaded!
				curLayout = MayronUI.db.profile.CurrentLayout;
				ADDONS = GetAceAddOns();
				local size = ( (#ADDONS < 6) and #ADDONS) or 6;				
				for i = 1, size do
					-- Check Boxes:
					local cb = CreateFrame("CheckButton", "MUI_AceAddon"..i, self, "UICheckButtonTemplate")
					if (i == 1) then
						cb:SetPoint("TOPLEFT", self, "TOPLEFT", 4, -4)
					else
						cb:SetPoint("TOPLEFT", "MUI_AceAddon"..(i-1), "BOTTOMLEFT", 0, 0)
					end
					
					-- FontString Names:
					_G["MUI_AceAddon"..i.."Text"]:SetText(ADDONS[i])
					_G["MUI_AceAddon"..i.."Text"]:SetPoint("LEFT", _G["MUI_AceAddon"..i], "RIGHT", 2, 0)

					-- DropDown Boxes for each Ace-AddOn:
					local dropdown = CreateFrame("Frame", "MUI_AceAddon"..i.."DropDown", self, "UIDropDownMenuTemplate")
					if (i == 1) then
						dropdown:SetPoint("TOPLEFT", _G[self:GetName().."ProfileHeader"], "BOTTOMLEFT", -16, -8)
					else
						dropdown:SetPoint("TOPLEFT", "MUI_AceAddon"..(i-1).."DropDown", "BOTTOMLEFT", 0, 0)
					end
					dropdown.name = ADDONS[i]; -- identifier (addon name)
					
					if (MayronUI.db.global.Layouts[curLayout][dropdown.name]) then
						_G["MUI_AceAddon"..i]:SetChecked(true)
					end
					
					-- OnLoad:
					if ( not _G["MUI_AceAddon"..i]:GetChecked() ) then
						_G["MUI_AceAddon"..i.."Text"]:SetFontObject("GameFontDisable")
						_G["MUI_AceAddon"..i.."DropDownButton"]:Disable()
					end
					
					-- Scripts:
					UIDropDownMenu_Initialize(dropdown, MUI_AddonDropDown_Initialize);
					UIDropDownMenu_SetAnchor(dropdown, 0, 0, "TOPRIGHT", _G["MUI_AceAddon"..i.."DropDownButton"], "BOTTOMRIGHT");			
					cb:SetScript("OnClick", function(self) 
						if ( self:GetChecked() ) then
							_G[self:GetName().."Text"]:SetFontObject("GameFontNormal")
							_G["MUI_AceAddon"..i.."DropDownButton"]:Enable()
							MayronUI.db.global.Layouts[curLayout][dropdown.name] = (GetDBObject(dropdown.name)):GetCurrentProfile();
						else
							_G[self:GetName().."Text"]:SetFontObject("GameFontDisable")
							_G["MUI_AceAddon"..i.."DropDownButton"]:Disable()
							MayronUI.db.global.Layouts[curLayout][dropdown.name] = nil;
						end
					end)
				end	
			end
			
			function MUI_LayoutAddonScrollBar_Update()
				FauxScrollFrame_Update(MUI_LayoutToolAddonsScrollFrame, #ADDONS, 6, 30);				
				local offset = FauxScrollFrame_GetOffset(MUI_LayoutToolAddonsScrollFrame);
				for i = 1, 6 do
					local cb = _G["MUI_AceAddon"..i];
					if (not cb) then break end
					local cbtxt = _G["MUI_AceAddon"..i.."Text"];					
					local dropdown = _G["MUI_AceAddon"..i.."DropDown"];	
					
					-- Assign Values here:
					cb.id = i + offset;	
					cb.value = cb.id;
					cbtxt:SetText(ADDONS[cb.id]);
					dropdown.name = ADDONS[cb.id];
					local path = GetDBObject(dropdown.name)
					
					-- Change State depending on AddOn included in Layout Switch:
					if (MayronUI.db.global.Layouts[curLayout][dropdown.name]) then
						UIDropDownMenu_SetText(dropdown, MayronUI.db.global.Layouts[curLayout][dropdown.name])			
						cb:SetChecked(true)
						cbtxt:SetFontObject("GameFontNormal")
						_G[dropdown:GetName().."Button"]:Enable()				
					else
						if (path and path.profile) then
							UIDropDownMenu_SetText(dropdown, path:GetCurrentProfile())
						else
							UIDropDownMenu_SetText(dropdown, "ERROR: "..dropdown.name.." failed to load!")
						end
						cb:SetChecked(false)
						cbtxt:SetFontObject("GameFontDisable")
						_G[dropdown:GetName().."Button"]:Disable()
					end
				end
			end
		end
	end -- End of Layout Switch AddOn!
	-- function to enable/disable chat frames per corner onload and used by the GUI:
	function MUI_ChatFrame_CornerSet()
		
		for corner, id in pairs(MayronUI.db.profile.ChatBox) do
			if (_G["MUI_ChatFrame"..id]) then _G["MUI_ChatFrame"..id]:Show() else
				local cf = CreateFrame("Frame", "MUI_ChatFrame"..id, UIParent, "MUI_ChatFrame")
				local sidebar = _G[cf.name.."SideBar"]				
				local tabs = _G[cf.name.."Tabs"]
				local anchor = corner:gsub("%s+", ""):upper();
				local mui = MayronUI.db.profile.ChatFrame[corner];
				
				-- Reflection Changes per corner:
				if (corner ~= "Top Left") then
					cf:ClearAllPoints()
					cf.Window:ClearAllPoints()
					sidebar:ClearAllPoints()
					_G[cf.name.."LeftButton"]:ClearAllPoints()
				end
				if (corner == "Top Right") then				
					tabs:SetTexCoord(1, 0, 0, 1)
					tabs:ClearAllPoints()
					tabs:SetPoint(anchor, sidebar, "TOPLEFT", 0, -12)	
					cf:SetPoint(anchor, UIParent, anchor, -2, -2)				
					sidebar:SetPoint(anchor, cf, anchor, 0 , -10)				
					cf.Window:SetPoint(anchor, tabs, "BOTTOMRIGHT", -2, -2)
					cf.Window.Texture:SetTexCoord(1, 0, 0, 1)				
					_G[cf.name.."LeftButton"]:SetPoint("BOTTOMLEFT", tabs, "TOPLEFT", -46, 2)
					sidebar:SetTexCoord(1, 0, 0, 1)
					_G[cf.name.."LayoutButton"]:GetNormalTexture():SetTexCoord(1, 0, 0, 1)
					_G[cf.name.."LayoutButton"]:SetPoint("LEFT", sidebar, "LEFT", 2, 0)		
				elseif (corner == "Bottom Left") then									
					cf:SetPoint(anchor, UIParent, anchor, 2, 2)				
					sidebar:SetPoint(anchor, cf, anchor, 0 , 10)				
					cf.Window:SetPoint("BOTTOMLEFT", sidebar, "BOTTOMRIGHT", 2, 12)
					cf.Window.Texture:SetTexCoord(0, 1, 1, 0)				
					_G[cf.name.."LeftButton"]:SetPoint("BOTTOMLEFT", sidebar, "BOTTOMRIGHT", 0, -10)
					_G[cf.name.."LeftButton"]:GetNormalTexture():SetTexCoord(0, 1, 1, 0)				
					_G[cf.name.."MiddleButton"]:GetNormalTexture():SetTexCoord(0, 1, 1, 0)
					_G[cf.name.."RightButton"]:GetNormalTexture():SetTexCoord(1, 0, 1, 0)
					
				elseif (corner == "Bottom Right") then
					tabs:SetTexCoord(1, 0, 0, 1)
					tabs:ClearAllPoints()
					tabs:SetPoint("TOPRIGHT", sidebar, "TOPLEFT", 0, -12)				
					cf:SetPoint(anchor, UIParent, anchor, -2, 2)				
					sidebar:SetPoint(anchor, cf, anchor, 0 , 10)				
					cf.Window:SetPoint(anchor, sidebar, "BOTTOMLEFT", -2, 12)
					cf.Window.Texture:SetTexCoord(1, 0, 1, 0)	
					_G[cf.name.."LeftButton"]:SetPoint("BOTTOMLEFT", cf.Window, "BOTTOMLEFT", -36, -22)
					_G[cf.name.."LeftButton"]:GetNormalTexture():SetTexCoord(0, 1, 1, 0)				
					_G[cf.name.."MiddleButton"]:GetNormalTexture():SetTexCoord(0, 1, 1, 0)
					_G[cf.name.."RightButton"]:GetNormalTexture():SetTexCoord(1, 0, 1, 0)
					sidebar:SetTexCoord(1, 0, 0, 1)
					_G[cf.name.."LayoutButton"]:GetNormalTexture():SetTexCoord(1, 0, 0, 1)
					_G[cf.name.."LayoutButton"]:SetPoint("LEFT", sidebar, "LEFT", 2, 0)
				end
				
				-- Setting alterations:
				if (mui.TabBackground) then
					_G[cf.name.."Tabs"]:Hide()
				end
				if (mui.BackgroundWindow) then
					_G[cf.name].Window:Hide()
				end
				if (mui.SideButton) then
					_G[cf.name.."LayoutButton"]:Hide()
				end
				if (mui.TabYOffset) then
					local p, rf, rp, x, _ = _G[cf.name.."Tabs"]:GetPoint()
					if ( mui.TabYOffset > -200 and mui.TabYOffset < -70) then
						if (corner:find("Right")) then x = 3 else x = -3 end
						_G[cf.name.."Tabs"]:SetPoint(p, rf, rp, x, mui.TabYOffset)
					else
						_G[cf.name.."Tabs"]:SetPoint(p, rf, rp, 0, mui.TabYOffset)
					end
				end
				if (mui.WindowYOffset) then
					local p, rf, rp, x, _ = _G[cf.name].Window:GetPoint()					
					_G[cf.name].Window:SetPoint(p, rf, rp, x, mui.WindowYOffset)
				end
				if (mui.WindowWidth) then				
					_G[cf.name].Window:SetWidth(mui.WindowWidth)
				end
				if (mui.WindowHeight) then				
					_G[cf.name].Window:SetHeight(mui.WindowHeight)
				end
			end
		end			
	end	
	if (not MayronUI.db.profile.Installed) then
		while not (next(MayronUI.db.profile.ChatBox)) do				
			MayronUI.db.profile.ChatBox = {
				["Top Left"] = 1,
			}
			break;
		end
	end
	MUI_ChatFrame_CornerSet();
	
	hooksecurefunc("BNToastFrame_Show", function()
		BNToastFrame:ClearAllPoints();
		if (MayronUI.db.profile.ChatFrame["Top Left"] and MUI_ChatFrame1) then
			BNToastFrame:SetPoint("TOPLEFT", MUI_ChatFrame1, "BOTTOMLEFT", 0, 0);
		elseif (MayronUI.db.profile.ChatFrame["Bottom Left"] and MUI_ChatFrame2) then
			BNToastFrame:SetPoint("BOTTOMLEFT", MUI_ChatFrame2, "TOPLEFT", 0, 0);
		end
	end)
	
	-- Bazooka Bar Setup:
	if ( (MayronUI.db.profile.ChatFrame.ReAnchor) and (IsAddOnLoaded("Bazooka")) ) then
		if ( (Bazooka.db:GetCurrentProfile() == "Default") and (not MayronUI.db.profile.ChatBox["Top Left"]) ) then
			if (MayronUI.db.profile.ChatBox["Bottom Left"]) then
				Bazooka.db:SetProfile("Bottom Left");							
			end
		elseif ( (Bazooka.db:GetCurrentProfile() == "Bottom Left") and (not MayronUI.db.profile.ChatBox["Bottom Left"]) ) then
			if (MayronUI.db.profile.ChatBox["Top Left"]) then
				Bazooka.db:SetProfile("Default");
			end
		end	
		local cbAnchor, profile, points;
		if ( (MayronUI.db.profile.ChatBox["Top Left"]) and (not MayronUI.db.profile.ChatBox["Bottom Left"]) ) then
			cbAnchor = MUI_ChatFrame1;
			profile = "Default";
			points = {-30, 16};
		elseif ( (MayronUI.db.profile.ChatBox["Bottom Left"]) and (not MayronUI.db.profile.ChatBox["Top Left"]) ) then
			cbAnchor = MUI_ChatFrame3;
			profile = "Bottom Left";
			points = {-16, 30};
		end
		if (cbAnchor and profile and points) then
			for i = 2, 4 do
				local bar = _G["BazookaBar_"..i];
				if (bar) then
					bar:ClearAllPoints();
					if (i == 2) then
						bar:SetPoint("TOPLEFT", cbAnchor, "TOPLEFT", 4, points[1]);
					elseif (i == 3) then
						bar:SetPoint("TOPLEFT", BazookaBar_2, "BOTTOMLEFT", 0, -3);
					else
						bar:SetPoint("TOPLEFT", BazookaBar_3, "BOTTOMLEFT", 0, -3);
					end
					local left = bar:GetLeft();
					local bottom = bar:GetBottom();
					BazookaDB["profiles"][profile]["bars"][i]["point"] = "BOTTOMLEFT";
					BazookaDB["profiles"][profile]["bars"][i]["relPoint"] = "BOTTOMLEFT";
					BazookaDB["profiles"][profile]["bars"][i]["y"] = bottom;
					BazookaDB["profiles"][profile]["bars"][i]["x"] = left;
				end
			end

			for i = 7, 5, -1 do
				local bar = _G["BazookaBar_"..i]
				if (bar) then
					bar:ClearAllPoints();
					if (i == 7) then
						bar:SetPoint("BOTTOMLEFT", cbAnchor, "BOTTOMLEFT", 2, points[2]);
					elseif (i == 6) then
						bar:SetPoint("BOTTOMLEFT", BazookaBar_7, "TOPLEFT", 2, 3);
					else
						bar:SetPoint("BOTTOMLEFT", BazookaBar_6, "TOPLEFT", 0, 3);
					end
					local left = bar:GetLeft();
					local bottom = bar:GetBottom();
					BazookaDB["profiles"][profile]["bars"][i]["point"] = "BOTTOMLEFT";
					BazookaDB["profiles"][profile]["bars"][i]["relPoint"] = "BOTTOMLEFT";
					BazookaDB["profiles"][profile]["bars"][i]["y"] = bottom;
					BazookaDB["profiles"][profile]["bars"][i]["x"] = left;
				end
			end
		end
		MayronUI.db.profile.ChatFrame.ReAnchor = nil;
	end	
	
	if (MayronUI.db.profile.ChatBox["Top Left"]) then
		-- Alter Compact Raid Manager:
		local SideButton = CreateFrame("Button", "MUI_SideButton3", MUI_ChatFrame1, "MUI_SideButtonTemplate")
		SideButton:SetPoint("LEFT", MUI_ChatFrame1, "BOTTOMLEFT", -2, -10);
		SideButton:SetPoint("TOP", MUI_ChatFrame1, "BOTTOM", -2, -40);
		SideButton:GetNormalTexture():SetTexCoord(1, 0, 0, 1);	
		SideButton:RegisterEvent("PLAYER_ENTERING_WORLD");
		SideButton:RegisterEvent("GROUP_ROSTER_UPDATE");
		SideButton:EnableMouse(false);
		SideButton:SetWidth(16);

		CompactRaidFrameManager:SetPoint("TOPLEFT", UIParent, "TOPLEFT", -7, -140);
		CompactRaidFrameContainer:SetParent(UIParent)
		CompactRaidFrameManagerToggleButton:ClearAllPoints()
		CompactRaidFrameManagerToggleButton:SetPoint("RIGHT", SideButton, "RIGHT", -10, 0);
		CompactRaidFrameManagerToggleButton:SetWidth(SideButton:GetWidth() + 5);
		CompactRaidFrameManagerToggleButton:SetHeight(SideButton:GetHeight() - 10);		
		CompactRaidFrameManager:SetFrameStrata("MEDIUM");
		CompactRaidFrameManager:SetFrameLevel(50);
		
		SideButton:SetScript("OnEvent", function(self, event)				
			if ( (event == "PLAYER_ENTERING_WORLD") or (event == "GROUP_ROSTER_UPDATE") ) then
					if (event == "PLAYER_ENTERING_WORLD") then
						CompactRaidFrameManagerToggleButton:SetScript("OnEnter", function() 
							SideButton:GetFontString():SetTextColor(1, 1, 1, 1); 
						end)					
						CompactRaidFrameManagerToggleButton:SetScript("OnLeave", function()  
							SideButton:GetFontString():SetTextColor(MUI_COLOR.r, MUI_COLOR.g, MUI_COLOR.b, 1); 
						end)
					end
				if ((GetNumGroupMembers() > 0) and (not self.InGroup)) then
					CompactRaidFrameManager:SetAlpha(0)
					SideButton:SetText(">")
					SideButton:Show()
					self.InGroup = true;
					if (InCombatLockdown()) then return end 
					CompactRaidFrameManager:EnableMouse(false)
				elseif ((GetNumGroupMembers() == 0) and (self.InGroup)) then
					SideButton:SetText(">")
					SideButton:Hide()
					self.InGroup = nil;
				end
			end
		end)
		
		hooksecurefunc("CompactRaidFrameManager_Collapse", function()
			CompactRaidFrameManager:SetAlpha(0)
			SideButton:SetText(">");
			if (InCombatLockdown()) then return end
			CompactRaidFrameManager:EnableMouse(false)
		end)
		hooksecurefunc("CompactRaidFrameManager_Expand", function()
			CompactRaidFrameManager:SetAlpha(1)
			SideButton:SetText("<");
			if (InCombatLockdown()) then return end
			CompactRaidFrameManager:EnableMouse(true)
		end)
	end
	
end
-- Has to be loaded instantly
for i = 1, NUM_CHAT_WINDOWS do
	_G["ChatFrame" .. i]:SetClampRectInsets(0, 0, 0, 0);
end